command_dpaste:
    type: command
    name: dpaste
    description: Paste a schematic using a name. If a schematic was saved in a pool, the pool MUST be specified.
    usage: /dpaste (name) (pool)
    aliases:
        - dplace
    script:
        - if <context.args.size> > 2:
            - narrate "<&4>Too many arguments! (No spaces allowed!)"
            - stop
        - else if <context.args.size> < 1:
            - narrate "/dpaste (name) (pool)"
            - narrate ----------------------------------------------
            - narrate "Paste a schematic using a name."
            - narrate "If a schematic was saved in a "
            - narrate "pool, the pool MUST be specified."
            - narrate ----------------------------------------------
            - stop
        # Checks whether if it is a classic schematic, or a schematic used in structure generation
        # structure generation schematics (ones in a pool) are always saved at the center of itself
        - choose <context.args.size>:
            - default:
                - narrate "<&4>Something went wrong!"
            - case 1:
                - run dschematic_paste_classic_task def.name:<context.args.get[1]> player:<player>
            - case 2:
                - run dschematic_paste_pool_task def.name:<context.args.get[1]> def.pool:<context.args.get[2]> player:<player>

dschematic_paste_classic_task:
    type: task
    debug: false
    definitions: name
    script:
        - define player <player>
        - define location <[player].location>
        # Check if the schematic name exists within the saved schematics flag
        - if !<server.has_flag[dschematic.classic.<[name]>]>:
            - narrate "<&4><[name]> does not exist!"
            - stop
        # Paste the schematic
        - ~schematic load name:<[name]>
        - ~schematic paste name:<[name]> <[location]>
        - narrate "<&2>Pasted <[name]> at <[location].simple>"
        - schematic unload name:<[name]>


dschematic_paste_pool_task:
    type: task
    debug: false
    definitions: name|pool
    script:
        - define player <player>
        - define location <[player].location>
        # Check if the schematic pool exists within the saved schematic flag
        - if !<server.has_flag[dschematic.pools.<[pool]>]>:
            - narrate "<&4>The pool <&dq><[pool]><&dq> does not exist!"
            - stop
        # Check if the schematic name exists within the saved schematics flag
        - if !<server.has_flag[dschematic.pools.<[pool]>.<[name]>]>:
            - narrate "<&4>The name <&dq><[name]><&dq> does not exist inside of <&dq><[pool]><&dq>!"
            - stop
        # Paste the schematic
        - ~schematic load name:<[pool]>_<[name]>
        - ~schematic paste name:<[pool]>_<[name]> <[location]>
        - narrate "<&2>Pasted <[pool]>:<[name]> at <[location].simple>"
        - schematic unload name:<[pool]>_<[name]>
