command_dschematic:
    type: command
    name: dschematic
    description: Save a schematic using a name. Optionally add a pool name to bundle similar schematics together for dungeon generation.
    usage: /dschematic (name) [pool] [weight]
    aliases:
        - dschem
        - dsave
    script:
        - if <context.args.size> > 3:
            - narrate "<&4>Too many arguments! (No spaces allowed!)"
            - stop
        - else if <context.args.size> < 1:
            - narrate "/dschematic (name) [pool] [weight]"
            - narrate ----------------------------------------------
            - narrate "Save a schematic using a name."
            - narrate "Optionally add a pool name to bundle similar"
            - narrate "schematics together for dungeon generation."
            - narrate ----------------------------------------------
            - stop
        # Checks if the weight is a whole, positive, number that doesn't contain anything else
        - if !<context.args.get[3].abs.is_truthy> && <context.args.size> > 2:
            - narrate "<&4>Invalid weight given!"
            - stop
        # Checks if it should save as a classic schematic or a structure generation schematic with a pool
        # structure generation schematics (ones in a pool) are always saved at the center of itself
        - choose <context.args.size>:
            - default:
                - narrate "<&4>Something went wrong!"
            - case 1:
                - run dschematic_save_task_classic def.name:<context.raw_args> player:<player>
            - case 2:
                - run dschematic_save_task_pool def.name:<context.args.get[1]> def.pool:<context.args.get[2]> def.weight:100 player:<player>
            - case 3:
                - run dschematic_save_task_pool def.name:<context.args.get[1]> def.pool:<context.args.get[2]> def.weight:<context.args.get[3]> player:<player>


dschematic_save_task_classic:
    type: task
    debug: false
    definitions: name
    script:
        - define player <player>
        # Check if a location was selected by the player
        - if !<[player].has_flag[dschematic.cuboid]>:
            - narrate "<&4>No location was selected!"
            - stop
        # Save a schematic and flag the saved name
        - flag server dschematic.classic.<[name]>
        - ~schematic create name:<[name]> area:<[player].flag[dschematic.cuboid]> <[player].location>
        - ~schematic save name:<[name]> area:<[player].flag[dschematic.cuboid]>
        - narrate "<&e>Saved as <&dq><[name]><&dq>"
        - narrate "<&e><[player].flag[dschematic.cuboid].volume> blocks saved."
        - schematic unload name:<[name]>


dschematic_save_task_pool:
    type: task
    debug: false
    definitions: name|pool|weight
    script:
        - define player <player>
        - if !<[player].has_flag[dschematic.cuboid]>:
            - narrate "<&4>No location was selected!"
            - stop

        - define cuboid:<[player].flag[dschematic.cuboid]>
        - define cuboid_size <list[<[cuboid].size.x>|<[cuboid].size.y>|<[cuboid].size.z>]>
        - define cuboid_center <[cuboid].center>
        - define largest 0

        # Manually check the highest number instead of using the tag (for some reason????)
        # TODO: Double check on this, and replace with the `highest` tag if you can
        - foreach <[cuboid_size]> as:size:
            - if <[size]> > 64:
                - narrate "<&4>Selection cannot exceed 64 blocks on any AXIS."
                - stop
            - if <[size]> > <[largest]>:
                - define largest:<[size]>
        # Find all jigsaws nearby and remove ones that aren't in the cuboid
        # Also remove and unflag locations that are flagged as jigsaws, but are not actually jigsaws
        - define jigsaws <[cuboid_center].find_blocks_flagged[dschematic.jigsaw].within[<[largest]>]>
        - foreach <[jigsaws]> as:jigsaw:
            - if <[jigsaw].material.name> != jigsaw:
                - flag <[jigsaw]> dschematic.jigsaw:!
                - define jigsaws:<-:<[jigsaw]>
                - foreach next
            - if !<[jigsaw].is_within[<[cuboid]>]>:
                - define jigsaws:<-:<[jigsaw]>
        # This should never happen
        - foreach <[jigsaws]> as:jigsaw:
            - if !<[jigsaw].has_flag[dschematic.jigsaw.return]> || !<[jigsaw].has_flag[dschematic.jigsaw.call]>:
                - narrate "<&4>A saved jigsaw had neither a return name or a call name!"
                - narrate "<&4>Check the jigsaw located at <[jigsaw].simple>"
                - stop


        # After passing all the checks above, identifies each jigsaw flagged and saves it to the schematic
        # TODO: Add functionality to save flagged Candle Cakes (Much easier to add than jigsaws so don't procastinate on it)
        - define jigsaw_map:<map>
        - foreach <[jigsaws]> as:jigsaw:
            - define jigsaw_relative:<[jigsaw].sub[<[cuboid_center]>]>
            - define jigsaw_map:<[jigsaw_map].with[<[jigsaw_relative]>].as[<[jigsaw].flag[dschematic.jigsaw]>]>

        # flags the schematic to the server
        # There is no way to remove or delete schematics besides:
        # 1.    manually unflagging and deleting the relevant schematic files
        # 2.    overwriting the schematic with the same name and pool
        - flag server dschematic.pools.<[pool]>.<[name]>
        - flag server dschematic.pools.<[pool]>.<[name]>.weight:<[weight]>
        - flag server dschematic.pools.<[pool]>.<[name]>.jigsaws:<[jigsaw_map]>
        - flag server dschematic.pools.<[pool]>.<[name]>.center:<[cuboid_center]>

        # Save the schematic
        - ~schematic create name:<[pool]>_<[name]> area:<[cuboid]> <[cuboid_center]> entities flags
        - ~schematic save name:<[pool]>_<[name]> area:<[cuboid]> <[cuboid_center]> entities flags
        - narrate "<&e>Saved as <&dq><[name]><&dq> in pool <&dq><[pool]><&dq>"
        - narrate "<&e><[jigsaws].size> jigsaws saved in schematic."
        - narrate "<&e><[cuboid].volume> blocks saved."
        - schematic unload name:<[pool]>_<[name]>