######################################################
##                                                  ##
##        MAIN FILE FOR STRUCTURE GENERATION        ##
##                                                  ##
######################################################

# This file is NOT configurable. It handles all structure generation.

dschematic_generate_structure_task:
    type: task
    debug: true
    definitions: start_name|pool_name|size|location
    script:
        - define player <player>
        - define location <[location].round_down>
        - if !<server.has_flag[dschematic.pools.<[pool_name]>]>:
            - narrate "<&4>The pool <&dq><[pool_name]><&dq> does not exist!"
            - stop
        - if !<server.has_flag[dschematic.pools.<[pool_name]>.<[start_name]>]>:
            - narrate "<&4>The name <&dq><[start_name]><&dq> does not exist inside of <&dq><[pool_name]><&dq>!"
            - stop
        - if <[size]> > 100 || <[size]> < 0:
            - narrate "<&4>Size must be between 0 to 100!"
            - stop
        - if !<[location].add[1,0,1].is_truthy>:
            - narrate "<&4>An invalid location was selected!"
            - stop
        # Prepares the pool, and what schematics to allow to be picked from
        - define pool <server.flag[dschematic.pools].get[<[pool_name]>]>
        - define max:0
        - foreach <[pool]> as:schematic:
            - define max:+:<[schematic].get[weight]>
        # Generates schematic and adds schematic name to a list, so it can be unloaded after the structure generation
        # also gets all jigsaws that were flagged in the schematic
        - define schematic_list:->:<[pool_name]>_<[start_name]>
        - ~schematic load name:<[pool_name]>_<[start_name]>
        - ~schematic paste name:<[pool_name]>_<[start_name]> <[location]>
        - define jigsaws <[pool].get[<[start_name]>].get[jigsaws]>
        - define keys <[jigsaws].keys>

        # Commence bullshit
        # Attempts to place a schematic that connects to each flagged jigsaw in the schematic placed above
        - foreach <[jigsaws]> as:jigsaw:
            # Finds the return jigsaw and defines accordingly
            - define jigsaw_return <[keys].get[<[loop_index]>].as[location]>
            - define jigsaw_return_location <[jigsaw_return].add[<[location].round_down>]>
            - define jigsaw_return_direction:<[jigsaw_return_location].block_facing>
            - define jigsaw_return_location_forward <[jigsaw_return_location].add[<[jigsaw_return_direction].with_y[0]>]>
            - choose <[jigsaw_return_direction].x>:
                - case 1:
                    - define jigsaw_return_yaw:270
                - case -1:
                    - define jigsaw_return_yaw:90
            - choose <[jigsaw_return_direction].z>:
                - case 1:
                    - define jigsaw_return_yaw:360
                - case -1:
                    - define jigsaw_return_yaw:180
            # Pick a schematic and a call jigsaw within that schematic to randomly attach to the return jigsaw
            - define next_schematic:<[pool].proc[dschematic_get_random_schematic]>
            - define schematic_to_paste:<[next_schematic].get[1]>
            - define schematic_name:<[next_schematic].get[2]>
            - define jigsaw_call <[schematic_to_paste].get[jigsaws].keys.random>
            - define jigsaw_call_facing <[schematic_to_paste].get[jigsaws].get[<[jigsaw_call]>].get[direction].before[_]>
            - choose <[jigsaw_call_facing]>:
                - case EAST:
                    - define jigsaw_call_yaw:270
                - case WEST:
                    - define jigsaw_call_yaw:90
                - case NORTH:
                    - define jigsaw_call_yaw:180
                - case SOUTH:
                    - define jigsaw_call_yaw:360
            # Define the rotation needed so the jigsaws are properly facing each other
            # Rotations are done in radians for locations not directly attached to the schematic
            ## THIS DOESN'T ALWAYS WORK, ESPECIALLY WITH T JUNCTIONS
            - choose <[jigsaw_return_yaw].sub[<[jigsaw_call_yaw]>]>:
                - case -270:
                    - define rotation:1.5707963268
                    - define schem_rotation:90
                - case -180:
                    - define rotation:0
                    - define schem_rotation:0
                - case -90:
                    - define rotation:4.71239
                    - define schem_rotation:270
                - case 0:
                    - define rotation:3.14159
                    - define schem_rotation:180
                - case 90:
                    - define rotation:1.5707963268
                    - define schem_rotation:90
                - case 180:
                    - define rotation:0
                    - define schem_rotation:0
                - case 270:
                    - define rotation:-1.5707963268
                    - define schem_rotation:-90
            # Get ready to place the conjoining structure
            - define jigsaw_call_location:<[jigsaw_call].as[location]>
            - define new_call_location:<[jigsaw_call_location].rotate_around_y[<[rotation]>].round_to_precision[0.5]>
            ## ALL DEBUG INFO HERE
            ################################################################
            - narrate <[loop_index]>
            - narrate ------------------------------------------------
            - narrate "RETURN FORWARD <[jigsaw_return_location_forward]>"
            - narrate "Rotational Difference <[jigsaw_return_yaw].sub[<[jigsaw_call_yaw]>]>"
            - narrate "Rotation needed <[schem_rotation]>"
            - narrate "^^^^ in radians <[rotation]>"
            - narrate "CALL RELATIVE <[jigsaw_call_location]>"
            - narrate "CALL ROTATED <[jigsaw_call_location].rotate_around_y[<[rotation]>].round_to_precision[0.5]>"
            - narrate "CENTER PLACEMENT <[jigsaw_return_location_forward].sub[<[new_call_location]>]>"
            - narrate ------------------------------------------------
            ################################################################
            ## ALL DEBUG INFO HERE

            # Place the structure rotated, and displaced based on the jigsaws offset from the center of the schematic.
            # Rotates both the jigsaw and the structure according to the rotation data above (this can fuck up)
            # - I feel like there is a better way to do this almost, instead of what I did above...
            - if !<[schematic_list].contains[<[pool_name]>_<[schematic_name]>]>:
                - define schematic_list:->:<[pool_name]>_<[schematic_name]>
                - ~schematic load name:<[pool_name]>_<[schematic_name]>
                - ~schematic paste name:<[pool_name]>_<[schematic_name]> angle:<[schem_rotation]> <[jigsaw_return_location_forward].sub[<[new_call_location]>]>
            - else:
                - ~schematic paste name:<[pool_name]>_<[schematic_name]> angle:<[schem_rotation]> <[jigsaw_return_location_forward].sub[<[new_call_location]>]>
        # Unload all used schematics in the generation
        - foreach <[schematic_list]> as:schematic:
            - ~schematic unload name:<[schematic]>